@@ -29,6 +29,7 @@ the second, the ROPath iterator is put into tar block form.
 
 import cStringIO, types
 from duplicity import statistics
+from duplicity import util
 from duplicity.path import * #@UnusedWildImport
 from duplicity.lazy import * #@UnusedWildImport
 
@@ -121,7 +122,7 @@ def get_delta_path(new_path, sig_path, sigTarFile = None):
         ti.name = "signature/" + "/".join(index)
         sigTarFile.addfile(ti, cStringIO.StringIO(sig_string))
 
-    if new_path.isreg() and sig_path and sig_path.difftype == "signature":
+    if new_path.isreg() and sig_path and sig_path.isreg() and sig_path.difftype == "signature":
         delta_path.difftype = "diff"
         old_sigfp = sig_path.open("rb")
         newfp = FileWithReadCounter(new_path.open("rb"))
@@ -181,15 +182,17 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
     """
     collated = collate2iters(new_iter, sig_iter)
     if sig_fileobj:
-        sigTarFile = tarfile.TarFile("arbitrary", "w", sig_fileobj)
+        sigTarFile = util.make_tarfile("w", sig_fileobj)
     else:
         sigTarFile = None
     for new_path, sig_path in collated:
         log.Debug(_("Comparing %s and %s") % (new_path and new_path.index,
                                               sig_path and sig_path.index))
         if not new_path or not new_path.type:
-            # file doesn't exist
-            if sig_path and sig_path.exists():
+            # File doesn't exist (but ignore attempts to delete base dir;
+            # old versions of duplicity could have written out the sigtar in
+            # such a way as to fool us; LP: #929067)
+            if sig_path and sig_path.exists() and sig_path.index != ():
                 # but signature says it did
                 log.Info(_("D %s") %
                          (sig_path.get_relative_path(),),
@@ -224,16 +227,17 @@ def sigtar2path_iter(sigtarobj):
     """
     Convert signature tar file object open for reading into path iter
     """
-    tf = tarfile.TarFile("Arbitrary Name", "r", sigtarobj)
-    tf.debug = 2
+    tf = util.make_tarfile("r", sigtarobj)
+    tf.debug = 1
     for tarinfo in tf:
+        tiname = util.get_tarinfo_name(tarinfo)
         for prefix in ["signature/", "snapshot/", "deleted/"]:
-            if tarinfo.name.startswith(prefix):
-                # strip prefix and from name and set it to difftype
-                name, difftype = tarinfo.name[len(prefix):], prefix[:-1]
+            if tiname.startswith(prefix):
+                # strip prefix and '/' from name and set it to difftype
+                name, difftype = tiname[len(prefix):], prefix[:-1]
                 break
         else:
-            raise DiffDirException("Bad tarinfo name %s" % (tarinfo.name,))
+            raise DiffDirException("Bad tarinfo name %s" % (tiname,))
 
         index = tuple(name.split("/"))
         if not index[-1]:
@@ -464,16 +468,12 @@ class TarBlockIter:
         self.remember_value = None          # holds index of next block
         self.remember_block = None          # holds block of next block
 
-        # We need to instantiate a dummy TarFile just to get access to
-        # some of the functions like _get_full_headers.
-        self.tf = tarfile.TarFromIterator(None)
-
     def tarinfo2tarblock(self, index, tarinfo, file_data = ""):
         """
         Make tarblock out of tarinfo and file data
         """
         tarinfo.size = len(file_data)
-        headers = self.tf._get_full_headers(tarinfo)
+        headers = tarinfo.tobuf()
         blocks, remainder = divmod(tarinfo.size, tarfile.BLOCKSIZE) #@UnusedVariable
         if remainder > 0:
             filler_data = "\0" * (tarfile.BLOCKSIZE - remainder)
