@@ -60,7 +60,7 @@ class TempPath(path.Path):
         return fh
 
 
-def get_fileobj_duppath(dirpath, partname, permname, remname):
+def get_fileobj_duppath(dirpath, partname, permname, remname, overwrite=False):
     """
     Return a file object open for writing, will write to filename
 
@@ -76,7 +76,10 @@ def get_fileobj_duppath(dirpath, partname, permname, remname):
                            partname = partname, permname = permname, remname = remname)
     else:
         dp = path.DupPath(dirpath.name, index = (partname,))
-        fh = FileobjHooked(dp.filtered_open("ab"), tdp = None, dirpath = dirpath,
+        mode = "ab"
+        if overwrite:
+            mode = "wb"
+        fh = FileobjHooked(dp.filtered_open(mode), tdp = None, dirpath = dirpath,
                            partname = partname, permname = permname, remname = remname)
 
     def rename_and_forget():
@@ -181,11 +184,7 @@ class FileobjHooked:
             gpg.GPGWriteFile(src_iter, tgt.name, globals.gpg_profile, size = sys.maxint)
         else:
             os.system("cp -p \"%s\" \"%s\"" % (src.name, tgt.name))
-        globals.backend.put(tgt) #@UndefinedVariable
-        try:
-            util.ignore_missing(os.unlink, tgt.name)
-        except Exception, e:
-            log.Warn(_("Unable to delete %s: %s" % (tgt.name, str(e))))
+        globals.backend.move(tgt) #@UndefinedVariable
 
     def to_final(self):
         """
@@ -207,6 +206,18 @@ class FileobjHooked:
         """
         return self.fileobj.read(length)
 
+    def tell(self):
+        """
+        Returns current location of fileobj
+        """
+        return self.fileobj.tell()
+
+    def seek(self, offset):
+        """
+        Seeks to a location of fileobj
+        """
+        return self.fileobj.seek(offset)
+
     def close(self):
         """
         Close fileobj, running hooks right afterwards
