@@ -21,13 +21,16 @@
 
 """
 duplicity's gpg interface, builds upon Frank Tobin's GnuPGInterface
+which is now patched with some code for iterative threaded execution
+see duplicity's README for details
 """
 
 import os, types, tempfile, re, gzip
 
 from duplicity import misc
 from duplicity import globals
-from duplicity import GnuPGInterface
+from duplicity import gpginterface
+from duplicity import tempdir
 
 try:
     from hashlib import sha1
@@ -38,9 +41,6 @@ except ImportError:
 
 blocksize = 256 * 1024
 
-# user options appended by --gpg-options
-gpg_options = ""
-
 
 class GPGError(Exception):
     """
@@ -65,11 +65,11 @@ class GPGProfile:
         'AA0E73D2'.
         """
         assert passphrase is None or type(passphrase) is types.StringType
-        if sign_key:
-            assert recipients # can only sign with asym encryption
 
         self.passphrase = passphrase
+        self.signing_passphrase = passphrase
         self.sign_key = sign_key
+        self.encrypt_secring = None
         if recipients is not None:
             assert type(recipients) is types.ListType # must be list, not tuple
             self.recipients = recipients
@@ -96,48 +96,76 @@ class GPGFile:
         """
         self.status_fp = None # used to find signature
         self.closed = None # set to true after file closed
-        self.logger_fp = tempfile.TemporaryFile()
-        self.stderr_fp = tempfile.TemporaryFile()
+        self.logger_fp = tempfile.TemporaryFile( dir=tempdir.default().dir() )
+        self.stderr_fp = tempfile.TemporaryFile( dir=tempdir.default().dir() )
+        self.name = encrypt_path
+        self.byte_count = 0
 
         # Start GPG process - copied from GnuPGInterface docstring.
-        gnupg = GnuPGInterface.GnuPG()
+        gnupg = gpginterface.GnuPG()
         gnupg.options.meta_interactive = 0
         gnupg.options.extra_args.append('--no-secmem-warning')
         if globals.use_agent:
             gnupg.options.extra_args.append('--use-agent')
-        if gpg_options:
-            for opt in gpg_options.split():
+        if globals.gpg_options:
+            for opt in globals.gpg_options.split():
                 gnupg.options.extra_args.append(opt)
 
+        cmdlist = []
         if profile.sign_key:
             gnupg.options.default_key = profile.sign_key
+            cmdlist.append("--sign")
+        # encrypt: sign key needs passphrase
+        # decrypt: encrypt key needs passphrase
+        # special case: allow different symmetric pass with empty sign pass
+        if encrypt and profile.sign_key and profile.signing_passphrase:
+            passphrase = profile.signing_passphrase
+        else:
+            passphrase = profile.passphrase
+        # in case the passphrase is not set, pass an empty one to prevent
+        # TypeError: expected a character buffer object on .write()
+        if passphrase is None:
+            passphrase = ""
 
         if encrypt:
             if profile.recipients:
                 gnupg.options.recipients = profile.recipients
-                cmdlist = ['--encrypt']
-                if profile.sign_key:
-                    cmdlist.append("--sign")
+                cmdlist.append('--encrypt')
             else:
-                cmdlist = ['--symmetric']
+                cmdlist.append('--symmetric')
                 # use integrity protection
                 gnupg.options.extra_args.append('--force-mdc')
-            p1 = gnupg.run(cmdlist, create_fhs=['stdin', 'passphrase'],
+            # Skip the passphrase if using the agent
+            if globals.use_agent:
+                gnupg_fhs = ['stdin',]
+            else:
+                gnupg_fhs = ['stdin','passphrase']
+            p1 = gnupg.run(cmdlist, create_fhs=gnupg_fhs,
                            attach_fhs={'stdout': encrypt_path.open("wb"),
                                        'stderr': self.stderr_fp,
                                        'logger': self.logger_fp})
-            p1.handles['passphrase'].write(profile.passphrase)
-            p1.handles['passphrase'].close()
+            if not(globals.use_agent):
+                p1.handles['passphrase'].write(passphrase)
+                p1.handles['passphrase'].close()
             self.gpg_input = p1.handles['stdin']
         else:
-            self.status_fp = tempfile.TemporaryFile()
-            p1 = gnupg.run(['--decrypt'], create_fhs=['stdout', 'passphrase'],
+            if profile.recipients and profile.encrypt_secring:
+                cmdlist.append('--secret-keyring')
+                cmdlist.append(profile.encrypt_secring)
+            self.status_fp = tempfile.TemporaryFile( dir=tempdir.default().dir() )
+            # Skip the passphrase if using the agent
+            if globals.use_agent:
+                gnupg_fhs = ['stdout',]
+            else:
+                gnupg_fhs = ['stdout','passphrase']
+            p1 = gnupg.run(['--decrypt'], create_fhs=gnupg_fhs,
                            attach_fhs={'stdin': encrypt_path.open("rb"),
                                        'status': self.status_fp,
                                        'stderr': self.stderr_fp,
                                        'logger': self.logger_fp})
-            p1.handles['passphrase'].write(profile.passphrase)
-            p1.handles['passphrase'].close()
+            if not(globals.use_agent):
+                p1.handles['passphrase'].write(passphrase)
+                p1.handles['passphrase'].close()
             self.gpg_output = p1.handles['stdout']
         self.gpg_process = p1
         self.encrypt = encrypt
@@ -145,6 +173,8 @@ class GPGFile:
     def read(self, length = -1):
         try:
             res = self.gpg_output.read(length)
+            if res is not None:
+                self.byte_count += len(res)
         except Exception:
             self.gpg_failed()
         return res
@@ -152,10 +182,21 @@ class GPGFile:
     def write(self, buf):
         try:
             res = self.gpg_input.write(buf)
-        except:
+            if res is not None:
+                self.byte_count += len(res)
+        except Exception:
             self.gpg_failed()
         return res
 
+    def tell(self):
+        return self.byte_count
+
+    def seek(self, offset):
+        assert not self.encrypt
+        assert offset >= self.byte_count, "%d < %d" % (offset, self.byte_count)
+        if offset > self.byte_count:
+            self.read(offset - self.byte_count)
+
     def gpg_failed(self):
         msg = "GPG Failed, see log below:\n"
         msg += "===== Begin GnuPG log =====\n"
@@ -173,13 +214,13 @@ class GPGFile:
         if self.encrypt:
             try:
                 self.gpg_input.close()
-            except:
+            except Exception:
                 self.gpg_failed()
             if self.status_fp:
                 self.set_signature()
             try:
                 self.gpg_process.wait()
-            except:
+            except Exception:
                 self.gpg_failed()
         else:
             res = 1
@@ -187,17 +228,17 @@ class GPGFile:
                 # discard remaining output to avoid GPG error
                 try:
                     res = self.gpg_output.read(blocksize)
-                except:
+                except Exception:
                     self.gpg_failed()
             try:
                 self.gpg_output.close()
-            except:
+            except Exception:
                 self.gpg_failed()
             if self.status_fp:
                 self.set_signature()
             try:
                 self.gpg_process.wait()
-            except:
+            except Exception:
                 self.gpg_failed()
         self.logger_fp.close()
         self.stderr_fp.close()
@@ -348,7 +389,7 @@ def get_hash(hash, path, hex = 1):
     hash should be "MD5" or "SHA1".  The output will be in hexadecimal
     form if hex is true, and in text (base64) otherwise.
     """
-    assert path.isreg()
+    #assert path.isreg()
     fp = path.open("rb")
     if hash == "SHA1":
         hash_obj = sha1()
