@@ -3,6 +3,7 @@
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
 # Copyright 2008 Michael Terry <mike@mterry.name>
+# Copyright 2011 Canonical Ltd
 #
 # This file is part of duplicity.
 #
@@ -46,6 +47,14 @@ def LoggerToDupLevel(verb):
        more severe"""
     return DupToLoggerLevel(verb)
 
+def LevelName(level):
+    level = LoggerToDupLevel(level)
+    if   level >= 9: return "DEBUG"
+    elif level >= 5: return "INFO"
+    elif level >= 3: return "NOTICE"
+    elif level >= 1: return "WARNING"
+    else:            return "ERROR"
+
 def Log(s, verb_level, code=1, extra=None, force_print=False):
     """Write s to stderr if verbosity level low enough"""
     global _logger
@@ -128,6 +137,8 @@ class WarningCode:
     cannot_stat = 9
     cannot_read = 10
     no_sig_for_time = 11
+    cannot_process = 12
+    process_skipped = 13
 
 def Warn(s, code=WarningCode.generic, extra=None):
     """Shortcut used for warning messages (verbosity 2)"""
@@ -179,8 +190,22 @@ class ErrorCode:
     restart_file_not_found = 39
     gio_not_available = 40
     source_dir_mismatch = 42 # 41 is reserved for par2
+    ftps_lftp_missing = 43
+    volume_wrong_size = 44
+    enryption_mismatch = 45
+    pythonoptimize_set = 46
 
-def FatalError(s, code, extra=None):
+    # 50->69 reserved for backend errors
+    backend_error = 50
+    backend_permission_denied = 51
+    backend_not_found = 52
+    backend_no_space = 53
+
+    # Reserve 126 because it is used as an error code for pkexec
+    # Reserve 127 because it is used as an error code for pkexec
+    # Reserve 255 because it is used as an error code for gksu
+
+def FatalError(s, code=ErrorCode.generic, extra=None):
     """Write fatal error message and exit"""
     Log(s, ERROR, code, extra)
     shutdown()
@@ -192,6 +217,7 @@ class DupLogRecord(logging.LogRecord):
         global _logger
         logging.LogRecord.__init__(self, *args, **kwargs)
         self.controlLine = controlLine
+        self.levelName = LevelName(self.levelno)
 
 class DupLogger(logging.Logger):
     """Custom logger that creates special code-bearing records"""
@@ -220,18 +246,6 @@ def setup():
     logging.setLoggerClass(DupLogger)
     _logger = logging.getLogger("duplicity")
 
-    # Set up our special level names
-    logging.addLevelName(DupToLoggerLevel(0), "ERROR")
-    logging.addLevelName(DupToLoggerLevel(1), "WARNING")
-    logging.addLevelName(DupToLoggerLevel(2), "WARNING")
-    logging.addLevelName(DupToLoggerLevel(3), "NOTICE")
-    logging.addLevelName(DupToLoggerLevel(4), "NOTICE")
-    logging.addLevelName(DupToLoggerLevel(5), "INFO")
-    logging.addLevelName(DupToLoggerLevel(6), "INFO")
-    logging.addLevelName(DupToLoggerLevel(7), "INFO")
-    logging.addLevelName(DupToLoggerLevel(8), "INFO")
-    logging.addLevelName(DupToLoggerLevel(9), "DEBUG")
-
     # Default verbosity allows notices and above
     setverbosity(NOTICE)
 
@@ -249,7 +263,11 @@ class MachineFormatter(logging.Formatter):
        processes."""
     def __init__(self):
         # 'message' will be appended by format()
-        logging.Formatter.__init__(self, "%(levelname)s %(controlLine)s")
+        # Note that we use our own, custom-created 'levelName' instead of the
+        # standard 'levelname'.  This is because the standard 'levelname' can
+        # be adjusted by any library anywhere in our stack without us knowing.
+        # But we control 'levelName'.
+        logging.Formatter.__init__(self, "%(levelName)s %(controlLine)s")
 
     def format(self, record):
         s = logging.Formatter.format(self, record)
@@ -265,13 +283,8 @@ class MachineFormatter(logging.Formatter):
 class MachineFilter(logging.Filter):
     """Filter that only allows levels that are consumable by other processes."""
     def filter(self, record):
-        # We only want to allow records that have level names.  If the level
-        # does not have an associated name, there will be a space as the
-        # logging module expands levelname to "Level %d".  This will confuse
-        # consumers.  Even if we dropped the space, random levels may not
-        # mean anything to consumers.
-        s = logging.getLevelName(record.levelno)
-        return s.find(' ') == -1
+        # We only want to allow records that have our custom level names
+        return hasattr(record, 'levelName')
 
 def add_fd(fd):
     """Add stream to which to write machine-readable logging"""
@@ -284,7 +297,7 @@ def add_fd(fd):
 def add_file(filename):
     """Add file to which to write machine-readable logging"""
     global _logger
-    handler = logging.FileHandler(filename, 'w')
+    handler = logging.FileHandler(filename)
     handler.setFormatter(MachineFormatter())
     handler.addFilter(MachineFilter())
     _logger.addHandler(handler)
