@@ -31,6 +31,7 @@ import threading
 import tempfile
 
 from duplicity import log
+from duplicity import util
 from duplicity import globals
 
 # Set up state related to managing the default temporary directory
@@ -126,6 +127,12 @@ class TemporaryDirectory:
 
         self.__lock = threading.Lock()  # protect private resources *AND* mktemp/mkstemp calls
 
+    def dir(self):
+    	"""
+    	Returns the absolute pathname of the temp folder.
+    	"""
+        return self.__dir 
+
     def __del__(self):
         """
         Perform cleanup.
@@ -230,13 +237,13 @@ class TemporaryDirectory:
                 for file in self.__pending.keys():
                     try:
                         log.Debug(_("Removing still remembered temporary file %s") % (file,))
-                        os.unlink(file)
-                    except:
+                        util.ignore_missing(os.unlink, file)
+                    except Exception:
                         log.Info(_("Cleanup of temporary file %s failed") % (file,))
                         pass
                 try:
                     os.rmdir(self.__dir)
-                except:
+                except Exception:
                     log.Warn(_("Cleanup of temporary directory %s failed - this is probably a bug.") % (self.__dir,))
                     pass
                 self.__pending = None
